@@ -31,7 +31,7 @@
 #define PRINTERVAL 20 /** Print Progress Interval (seconds) */
 #define CRASHFILE "savedstate.sav"
 #define VERSION_MAJOR 0
-#define VERSION_MINOR 9
+#define VERSION_MINOR 10
 
 /** alarmInterrupt is used to print out the progress at specific intervals */
 static void
@@ -252,17 +252,13 @@ main(int argc, char** argv) {
     if(ret) {
       if(ret == EENCNF) 
 	fprintf(stderr, "Error: Could not extract encryption information\n");
-      else if(ret == ETRANF)
-	fprintf(stderr, "Error: First trailer not found\n");
-      else if(ret == ETRENF)
-	fprintf(stderr, "Error: Encryption object not found in trailer\n");
-      else if(ret == ETRINF)
-	fprintf(stderr, "Error: ID object not found in trailer\n");
+      else if(ret == ETRANF || ret == ETRENF || ret == ETRINF)
+	fprintf(stderr, "Error: Encryption not detected (is the document password protected?)\n");
       ret = 4;
       goto out1;
     }
     else if(e->revision < 2 || (strcmp(e->s_handler,"Standard") != 0)) {
-      fprintf(stderr, "The specific version is not supported\n");
+      fprintf(stderr, "The specific version is not supported (%s - %d)\n", e->s_handler, e->revision);
       ret = 5;
       goto out1;
       }
