@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2006 Henning Norén
+ * Copyright (C) 2006-2008 Henning Norén
  * Copyright (C) 1996-2005 Glyph & Cog, LLC.
  * 
  * This program is free software; you can redistribute it and/or
@@ -46,7 +46,7 @@ pad[32] = {
 
 /** buffers for stuff that we can precompute before the actual cracking */
 static uint8_t *encKeyWorkSpace;
-static uint8_t password_user[32];
+static uint8_t password_user[33];
 static uint8_t *rev3TestKey;
 static unsigned int ekwlen;
 
@@ -97,7 +97,7 @@ initEncKeyWorkSpace(const int revision, const bool encMetaData,
 		    const int permissions, const uint8_t *ownerkey,
 		    const uint8_t *fileID, const unsigned int fileIDLen) {
   /** 
-   *   Algorithm 3.2 Computing an encryption key (PDF Reference, v 1.6, p.101)
+   *   Algorithm 3.2 Computing an encryption key (PDF Reference, v 1.7, p.125)
    *
    *   Make space for:
    *   field           | bytes
@@ -108,7 +108,7 @@ initEncKeyWorkSpace(const int revision, const bool encMetaData,
    *   fileID          | <fileIDLEn>
    *  [extra padding]  | [4] (Special for step 6)
    **/
-  unsigned int size = (revision > 2 && !encMetaData) ? 72 : 68;
+  unsigned int size = (revision > 3 && !encMetaData) ? 72 : 68;
   encKeyWorkSpace = malloc(size + fileIDLen);
 
   /** Just to be sure we have no uninitalized stuff in the workspace */
@@ -127,7 +127,7 @@ initEncKeyWorkSpace(const int revision, const bool encMetaData,
   memcpy(encKeyWorkSpace + 68, fileID, fileIDLen);
 
   /** 6 */
-  if(revision > 2 && !encMetaData) {
+  if(revision > 3 && !encMetaData) {
     encKeyWorkSpace[68+fileIDLen] = 0xff;
     encKeyWorkSpace[69+fileIDLen] = 0xff;
     encKeyWorkSpace[70+fileIDLen] = 0xff;
@@ -195,14 +195,23 @@ static void
 foundPassword(void) {
   char str[33];
 
-  /** The only password that can be found in plaintext is the password pointed
-      to by currPW. If we for example would like to print out the userpassword
-      when we have found the correct ownerpassword we need to recalculate it
-      from this password
-  */
   memcpy(str,currPW,currPWLen);
   str[currPWLen] = '\0';
   printf("found %s-password: '%s'\n", workWithUser?"user":"owner", str);
+
+  /** The only password that can be found in plaintext is the password pointed
+      to by currPW. If SHOWUSERPASSWORD is define we will get the password
+      via user_password _including_ the pad.
+      For this to be really usable we would print out the possible userpasswords
+      based on finding possible beginnings of the pad (there are real passwords
+      which can be used which matches the same as a padded (not that it really
+      matters)).
+  */
+
+#ifdef SHOWUSERPASSWORD
+  if(!workWithUser)
+    printf("found user-password: '%s'\n", password_user);
+#endif
 }
 
 /** Common handling of the key for all rev3-functions */
@@ -270,7 +279,7 @@ runCrackRev2_o(void) {
 
   lpasslength = 0;
   startTime = time(NULL);
-  while(nextPassword()) {
+  do {
     BEGIN_CRACK_LOOP();
   
     do {
@@ -278,12 +287,16 @@ runCrackRev2_o(void) {
 
       rc4Decrypt(enckey, encdata->o_string, 32, encKeyWorkSpace);
       md5(encKeyWorkSpace, ekwlen, enckey);
-      if(rc4Match40b(enckey, encdata->u_string, pad))
+      if(rc4Match40b(enckey, encdata->u_string, pad)) {
+#ifdef SHOWUSERPASSWORD
+	memcpy(password_user, encKeyWorkSpace, 32);
+#endif
 	return true;
+      }
 
       ++nrprocessed;
     } while(permutate());
-  }
+  } while(nextPassword());
   return false;
 }
 
@@ -296,7 +309,7 @@ runCrackRev3_o(void) {
   length = encdata->length/8;
   lpasslength = 0;
   startTime = time(NULL);
-  while(nextPassword()) {
+  do {
     BEGIN_CRACK_LOOP();
 
     do {
@@ -308,12 +321,16 @@ runCrackRev3_o(void) {
       RC4_DECRYPT_REV3(32);
       memcpy(encKeyWorkSpace, test, 32);
 
-      if(isUserPasswordRev3())
+      if(isUserPasswordRev3()) {
+#ifdef SHOWUSERPASSWORD
+        memcpy(password_user, encKeyWorkSpace, 32);
+#endif
 	return true;
+      }
 
       ++nrprocessed;
     } while(permutate());
-  }
+  } while(nextPassword());
   return false;
 }
 
@@ -324,7 +341,7 @@ runCrackRev2_of(void) {
 
   lpasslength = 0;
   startTime = time(NULL);
-  while(nextPassword()) {
+  do {
     BEGIN_CRACK_LOOP();
 
     do {
@@ -336,7 +353,7 @@ runCrackRev2_of(void) {
 
       ++nrprocessed;
     } while(permutate());
-  }
+  } while(nextPassword());
   return false;
 }
 
@@ -349,7 +366,7 @@ runCrackRev3_of(void) {
   length = encdata->length/8;
   lpasslength = 0;
   startTime = time(NULL);
-  while(nextPassword()) {
+  do {
     BEGIN_CRACK_LOOP();
     
     do {
@@ -370,7 +387,7 @@ runCrackRev3_of(void) {
 
       ++nrprocessed;
     } while(permutate());
-  }
+  } while(nextPassword());
   return false;
 }
 
@@ -383,7 +400,7 @@ runCrackRev3(void) {
   length = encdata->length/8;
   lpasslength = 0;
   startTime = time(NULL);
-  while(nextPassword()) {
+  do {
     BEGIN_CRACK_LOOP();
 
     do {
@@ -405,7 +422,7 @@ runCrackRev3(void) {
 
       ++nrprocessed;
     } while(permutate());
-  }
+  } while(nextPassword());
   return false;
 }
 
@@ -417,7 +434,7 @@ runCrackRev2(void) {
 
   lpasslength = 0;
   startTime = time(NULL);
-  while(nextPassword()) {
+  do {
     BEGIN_CRACK_LOOP();
 
     do {
@@ -429,7 +446,7 @@ runCrackRev2(void) {
 
       ++nrprocessed;
     } while(permutate());
-  }
+  } while(nextPassword());
   return false;
 }
 
