@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2006 Henning Norén
+ * Copyright (C) 2006-2008 Henning Norén
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,6 +25,13 @@
 
 /** Please rewrite all of this file in a clean and stable way */
 
+struct p_str {
+  uint8_t *content;
+  uint8_t len;
+};
+
+typedef struct p_str p_str;
+
 __attribute__ ((pure)) static inline bool
 isWhiteSpace(const int ch) {
   return (ch == 0x20 || (ch >= 0x09 && ch <= 0x0d) || ch == 0x00);
@@ -78,13 +85,13 @@ parseIntWithC(FILE *file, const int c) {
   return i;
 }
 
-static inline int
+static int
 parseInt(FILE *file) {
   return parseIntWithC(file, getc(file));
 }
 
 
-static inline char
+static char
 parseWhiteSpace(FILE *file) {
   int ch;
   do {
@@ -186,31 +193,38 @@ hexToInt(const int b) {
     return 0;
 }
 
-static uint8_t*
+static p_str*
 parseHexString(const uint8_t *buf, const unsigned int len) {
   unsigned int i,j;
-  uint8_t *ret;
-
-  ret = malloc(sizeof(uint8_t)*(len/2));
+  p_str *ret;
+ 
+  ret = malloc(sizeof(p_str));
+  ret->content = malloc(sizeof(uint8_t)*(len/2));
+  ret->len = (len/2);
 
   for(i=0, j=0; i<len; i += 2) {
-    ret[j] = hexToInt(buf[i]) * 16;
-    ret[j] += hexToInt(buf[i+1]);
+    ret->content[j] = hexToInt(buf[i]) * 16;
+    ret->content[j] += hexToInt(buf[i+1]);
     j++;
   }
+
   return ret;
 }
 
-static uint8_t*
+static p_str*
 objStringToByte(const uint8_t* str, const unsigned int len) {
-  unsigned int i, l;
+  unsigned int i, j, l;
   uint8_t b, d;
   uint8_t tmp[len];
-  uint8_t *ret;
+  p_str *ret;
 
   for(i=0, l=0; i<len; i++, l++) {
     b = str[i];
     if(b == '\\') {
+      /** 
+       * We have reached a special character or the beginning of a octal 
+       * up to three digit number and should skip the initial backslash
+       **/
       i++;
       switch(str[i]) {
       case 'n':
@@ -240,12 +254,20 @@ objStringToByte(const uint8_t* str, const unsigned int len) {
       default:
 	if(str[i] >= '0' && str[i] < '8') {
 	  d = 0;
-	  for(; i<len && 
+	  for(j=0; i < len && j < 3 &&
 		str[i] >= '0' && str[i] < '8' &&
-		(d*8)+str[i] < 256; i++) {
+		(d*8)+(str[i]-'0') < 256; j++, i++) {
 	    d *= 8;
 	    d += (str[i]-'0');
 	  }
+	  /** 
+	   * We need to step back one step if we reached the end of string
+	   * or the end of digits (like for example \0000)
+	   **/
+	  if(i < len || j < 3) {
+	    i--;
+	  }
+
 	  b = d;
 	}
       }
@@ -253,17 +275,20 @@ objStringToByte(const uint8_t* str, const unsigned int len) {
     tmp[l] = b;
   }
 
-  ret = malloc(sizeof(uint8_t)*l);
-  memcpy(ret, tmp, l);
-  
+  ret = malloc(sizeof(p_str));
+  ret->content = malloc(sizeof(uint8_t)*(l));
+  ret->len = l-1;
+
+  memcpy(ret->content, tmp, l);
+
   return ret;
 }
 
-static uint8_t*
+static p_str*
 parseRegularString(FILE *file) {
   unsigned int len, p;
   int ch;
-  uint8_t *ret;
+  p_str *ret;
   uint8_t buf[BUFFSIZE];
   bool skip = false;
 
@@ -310,11 +335,10 @@ static int
 findTrailer(FILE *file, EncData *e) {
   int ch;
   /**  int pos_i; */
-  unsigned int i = 0;
   bool encrypt = false;
   bool id = false;
   int e_pos = -1;
-  uint8_t buf[BUFFSIZE];
+  p_str *str = NULL;
   
   ch = getc(file);
   while(ch != EOF) {
@@ -357,34 +381,29 @@ findTrailer(FILE *file, EncData *e) {
 	      ch = parseWhiteSpace(file);
 	      while(ch != '[' && ch != EOF)
 		ch = getc(file);
-	      while(ch != '<' && ch != EOF)
-		ch = getc(file);
-	      i = 0;
-	      while(ch != '>' && i < BUFFSIZE && ch != EOF) {
-		if((ch >= '0' && ch <= '9') || 
-		   (ch >= 'a' && ch <= 'f') || 
-		   (ch >= 'A' && ch <= 'F')) {
-		  buf[i++] = ch;
-		}
-		ch = getc(file);
+
+	      if(str) {
+		if(str->content)
+		  free(str->content);
+		free(str);
 	      }
-	      if(ch == EOF)
-		break;
-	      if(i % 2 != 0)
-		buf[++i] = '0';
-/**
+
+	      str = parseRegularString(file);
+	      /**
 	      pos_i = ftell(file);
 	      printf("found ID at pos %x\n", pos_i);
 	      */
-	      id = true;
+	      if(str)
+		id = true;
 	      ch = getc(file);
 	    }
 	    else
 	      ch = getc(file);
 	    if(encrypt && id) {
 	      /**printf("found all, returning: epos: %d\n",e_pos);*/
-	      e->fileID = parseHexString(buf, i);
-	      e->fileIDLen = (i/2);
+	      e->fileID = str->content;
+	      e->fileIDLen = str->len;
+	      free(str);
 	      return e_pos;
 	    }
 	  }
@@ -399,8 +418,14 @@ findTrailer(FILE *file, EncData *e) {
   }
   /**  printf("finished searching\n");*/
 
+  if(str) {
+    if(str->content)
+      	free(str->content);
+    free(str);
+  }
+
   if(!encrypt && id)
-    return ETRENF;
+      return ETRENF;
   else if(!id && encrypt)
     return ETRINF;
   else 
@@ -418,6 +443,7 @@ parseEncrypObject(FILE *file, EncData *e) {
   bool fr = false;
   bool fu = false;
   bool fv = false;
+  p_str *str = NULL;
 
   ch = getc(file);
   while(ch != EOF) {
@@ -466,7 +492,12 @@ parseEncrypObject(FILE *file, EncData *e) {
 	}
 	break;
       case 'O':
-	e->o_string = parseRegularString(file);
+	str = parseRegularString(file);
+	if(str->len != 32)
+	  fprintf(stderr, "WARNING: O-String != 32 Bytes: %d\n", str->len);
+	e->o_string = str->content;
+     	if(str)
+	  free(str);
 	fo = true;
 	break;
       case 'P':
@@ -486,7 +517,12 @@ parseEncrypObject(FILE *file, EncData *e) {
 	}
 	break;
       case 'U':
-	e->u_string = parseRegularString(file);
+	str = parseRegularString(file);
+	if(str->len != 32)
+	  fprintf(stderr, "WARNING: U-String != 32 Bytes: %d\n", str->len);
+	e->u_string = str->content;
+	if(str)
+	  free(str);
 	fu = true;
 	break;
       case 'V':
@@ -552,17 +588,20 @@ findEncryptObject(FILE *file, const int e_pos, EncData *e) {
 
 int
 getEncryptedInfo(FILE *file, EncData *e) {
-  int e_pos;
+  int e_pos = -1;
   bool ret;
-  e_pos = findTrailer(file, e);
-  if(e_pos < 0)
-    return e_pos;
-  
-  ret = findEncryptObject(file, e_pos, e);
-  if(!ret) {
+
+  if(!fseek(file, 0L, SEEK_END-1024))
+    e_pos = findTrailer(file, e);
+  if(e_pos < 0) {
     rewind(file);
-    ret = findEncryptObject(file, e_pos, e);
+    e_pos = findTrailer(file, e);
   }
+  if(e_pos < 0) {
+    return e_pos;
+  }
+  rewind(file);
+  ret = findEncryptObject(file, e_pos, e);
   if(!ret)
     return EENCNF;
 
